@@ -44,6 +44,11 @@ COLUMN_BLACKLIST = ('_sa_polymorphic_on', )
 #: attributes of a model class.
 COLUMN_TYPES = (InstrumentedAttribute, hybrid_property)
 
+#: Strings which, when received by the server as the value of a date or time
+#: field, indicate that the server should use the current time when setting the
+#: value of the field.
+CURRENT_TIME_MARKERS = ('CURRENT_TIMESTAMP', 'CURRENT_DATE', 'LOCALTIMESTAMP')
+
 
 def partition(l, condition):
     """Returns a pair of lists, the left one containing all elements of `l` for
@@ -171,10 +176,13 @@ def is_date_field(model, fieldname):
     else:
         if isinstance(field, AssociationProxy):
             field = field.remote_attr
-        prop = field.property
-        if isinstance(prop, RelProperty):
+        if hasattr(field, 'property'):
+            prop = field.property
+            if isinstance(prop, RelProperty):
+                return False
+            fieldtype = prop.columns[0].type
+        else:
             return False
-        fieldtype = prop.columns[0].type
     return isinstance(fieldtype, Date) or isinstance(fieldtype, DateTime)
 
 
@@ -506,6 +514,8 @@ def strings_to_dates(model, dictionary):
         if is_date_field(model, fieldname) and value is not None:
             if value.strip() == '':
                 result[fieldname] = None
+            elif value in CURRENT_TIME_MARKERS:
+                result[fieldname] = getattr(func, value.lower())()
             else:
                 result[fieldname] = parse_datetime(value)
         else:
