@@ -28,6 +28,7 @@ from functools import wraps
 import math
 import warnings
 
+from flask import abort
 from flask import current_app
 from flask import json
 from flask import jsonify
@@ -37,8 +38,8 @@ from sqlalchemy.exc import IntegrityError
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.exc import MultipleResultsFound
 from sqlalchemy.orm.exc import NoResultFound
+from sqlalchemy.orm.query import Query
 
-from .exceptions import json_abort
 from .helpers import evaluate_functions
 from .helpers import get_by
 from .helpers import get_columns
@@ -148,6 +149,61 @@ def jsonify_status_code(status_code, headers=None, *args, **kw):
     return response
 
 
+# This code is (lightly) adapted from the ``requests`` library, in the
+# ``requests.utils`` module. See <http://python-requests.org> for more
+# information.
+def _link_to_json(value):
+    """Returns a list representation of the specified HTTP Link header
+    information.
+
+    `value` is a string containing the link header information. If the link
+    header information (the part of after ``Link:``) looked like this::
+
+        <url1>; rel="next", <url2>; rel="foo"; bar="baz"
+
+    then this function returns a list that looks like this::
+
+        [{"url": "url1", "rel": "next"},
+         {"url": "url2", "rel": "foo", "bar": "baz"}]
+
+    This example is adapted from the documentation of GitHub's API.
+
+    """
+    links = []
+    replace_chars = " '\""
+    for val in value.split(","):
+        try:
+            url, params = val.split(";", 1)
+        except ValueError:
+            url, params = val, ''
+        link = {}
+        link["url"] = url.strip("<> '\"")
+        for param in params.split(";"):
+            try:
+                key, value = param.split("=")
+            except ValueError:
+                break
+            link[key.strip(replace_chars)] = value.strip(replace_chars)
+        links.append(link)
+    return links
+
+
+def _headers_to_json(headers):
+    """Returns a dictionary representation of the specified dictionary of HTTP
+    headers ready for use as a JSON object.
+
+    Pre-condition: headers is not ``None``.
+
+    """
+    link = headers.pop('Link', None)
+    # Shallow copy is fine here because the `headers` dictionary maps strings
+    # to strings to strings.
+    result = headers.copy()
+    if link:
+        result['Link'] = _link_to_json(link)
+    return result
+
+
 def jsonpify(*args, **kw):
     """Passes the specified arguments directly to :func:`jsonify_status_code`
     with a status code of 200, then wraps the response with the name of a
@@ -161,16 +217,27 @@ def jsonpify(*args, **kw):
     dictionary.
 
     """
+    headers = kw.pop('headers', None)
     response = jsonify(*args, **kw)
     callback = request.args.get('callback', False)
     if callback:
-        content = '%s(%s)' % (callback, response.data)
+        # Reload the data from the constructed JSON string so we can wrap it in
+        # a JSONP function.
+        data = json.loads(response.data)
+        # Add the headers as metadata to the JSONP response.
+        meta = _headers_to_json(headers) if headers is not None else {}
+        # TODO add a jsonpify_status_code function?
+        meta['status'] = 200
+        inner = json.dumps(dict(meta=meta, data=data))
+        content = '%s( %s)' % (callback, inner)
         # Note that this is different from the mimetype used in Flask for JSON
-        # responses; Flask uses 'application/json'.
+        # responses; Flask uses 'application/json'. We use
+        # 'application/javascript' because a JSONP response is not valid JSON.
         mimetype = 'application/javascript'
         response = current_app.response_class(content, mimetype=mimetype)
-    if 'headers' in kw:
-        set_headers(response, kw['headers'])
+    # Set the headers on the HTTP response as well.
+    if headers:
+        set_headers(response, headers)
     return response
 
 
@@ -713,7 +780,10 @@ class API(ModelView):
            }
 
         """
-        num_results = len(instances)
+        if type(instances) == list:
+            num_results = len(instances)
+        else:
+            num_results = instances.count()
         results_per_page = self._compute_results_per_page()
         if results_per_page > 0:
             # get the page number (first page is page 1)
@@ -768,7 +838,7 @@ class API(ModelView):
         """
         inst = get_by(self.session, self.model, instid)
         if inst is None:
-            json_abort(404)
+            abort(404)
         return self._inst_to_dict(inst)
 
     def _search(self):
@@ -872,7 +942,7 @@ class API(ModelView):
         deep = dict((r, {}) for r in relations)
 
         # for security purposes, don't transmit list as top-level JSON
-        if isinstance(result, list):
+        if isinstance(result, Query):
             result = self._paginated(result, deep)
             # Create the Link header.
             #
@@ -899,7 +969,7 @@ class API(ModelView):
 
         return jsonpify(result, headers=headers)
 
-    def get(self, instid, relationname):
+    def get(self, instid, relationname, relationinstid):
         """Returns a JSON representation of an instance of model with the
         specified name.
 
@@ -920,7 +990,7 @@ class API(ModelView):
         # get the instance of the "main" model whose ID is instid
         instance = get_by(self.session, self.model, instid)
         if instance is None:
-            json_abort(404)
+            abort(404)
         # If no relation is requested, just return the instance. Otherwise,
         # get the value of the relation specified by `relationname`.
         if relationname is None:
@@ -940,7 +1010,7 @@ class API(ModelView):
             postprocessor(result=result)
         return jsonpify(result)
 
-    def delete(self, instid, relationname):
+    def delete(self, instid, relationname, relationinstid):
         """Removes the specified instance of the model with the specified name
         from the database.
 
@@ -948,17 +1018,33 @@ class API(ModelView):
         :rfc:`2616`, this method responds with :http:status:`204` regardless of
         whether an object was deleted.
 
-        This function ignores the `relationname` keyword argument.
+        If `relationname
 
-        .. versionadded:: 0.10
+        .. versionadded:: 0.12.0
+           Added the `relationinstid` keyword argument.
+
+        .. versionadded:: 0.10.0
            Added the `relationname` keyword argument.
 
         """
         is_deleted = False
         for preprocessor in self.preprocessors['DELETE']:
-            preprocessor(instance_id=instid)
+            preprocessor(instance_id=instid, relation_name=relationname,
+                         relation_instance_id=relationinstid)
         inst = get_by(self.session, self.model, instid)
-        if inst is not None:
+        if relationname:
+            # If the request is ``DELETE /api/person/1/computers``, error 400.
+            if not relationinstid:
+                msg = 'Cannot DELETE entire "%s" relation' % relationname
+                return jsonify_status_code(400, msg)
+            # Otherwise, get the related instance to delete.
+            relation = getattr(inst, relationname)
+            related_model = get_related_model(self.model, relationname)
+            relation_instance = get_by(self.session, related_model,
+                                       relationinstid)
+            # Removes an object from the relation list.
+            relation.remove(relation_instance)
+        elif inst is not None:
             self.session.delete(inst)
             self.session.commit()
             is_deleted = True
@@ -1065,7 +1151,7 @@ class API(ModelView):
             current_app.logger.exception(exception.message)
             return jsonify_status_code(400, message=exception.message)
 
-    def patch(self, instid, relationname):
+    def patch(self, instid, relationname, relationinstid):
         """Updates the instance specified by ``instid`` of the named model, or
         updates multiple instances if ``instid`` is ``None``.
 
@@ -1080,9 +1166,13 @@ class API(ModelView):
         parameters for restricting the set of instances on which updates will
         be made in this case.
 
-        This function ignores the `relationname` keyword argument.
+        This function ignores the `relationname` and `relationinstid` keyword
+        arguments.
 
-        .. versionadded:: 0.10
+        .. versionadded:: 0.12.0
+           Added the `relationinstid` keyword argument.
+
+        .. versionadded:: 0.10.0
            Added the `relationname` keyword argument.
 
         """
@@ -1130,7 +1220,7 @@ class API(ModelView):
             # create a SQLAlchemy Query which has exactly the specified row
             query = query_by_primary_key(self.session, self.model, instid)
             if query.count() == 0:
-                json_abort(404)
+                abort(404)
             assert query.count() == 1, 'Multiple rows with same ID'
 
         relations = self._update_relations(query, data)
@@ -1169,6 +1259,6 @@ class API(ModelView):
 
         return jsonify(result)
 
-    def put(self, instid, relationname):
+    def put(self, *args, **kw):
         """Alias for :meth:`patch`."""
-        return self.patch(instid, relationname)
+        return self.patch(*args, **kw)
